package gs.log.v0.data

import scala.reflect.ClassTag
import java.time.LocalDate
import java.time.Instant

trait DataRenderer:
  def render[A](data: A)(using CT: ClassTag[A]): String

object DataRenderer:

  abstract class SingleTypeRenderer[A]:
    def render(data: A): String

  object Defaults:

    final class StringRenderer extends SingleTypeRenderer[String]:
      override def render(data: String): String = data

    final class BooleanRenderer extends SingleTypeRenderer[Boolean]:
      override def render(data: Boolean): String = data.toString()

    final class IntRenderer extends SingleTypeRenderer[Int]:
      override def render(data: Int): String = data.toString()

    final class LongRenderer extends SingleTypeRenderer[Long]:
      override def render(data: Long): String = data.toString()

    final class FloatRenderer extends SingleTypeRenderer[Float]:
      override def render(data: Float): String = data.toString()

    final class DoubleRenderer extends SingleTypeRenderer[Double]:
      override def render(data: Double): String = data.toString()

    final class DateRenderer extends SingleTypeRenderer[LocalDate]:
      override def render(data: LocalDate): String = data.toString()

    final class InstantRenderer extends SingleTypeRenderer[Instant]:
      override def render(data: Instant): String = data.toString()

  end Defaults

  class RegistryRenderer(
    private val registry: Map[Class[?], SingleTypeRenderer[?]]
  ):

    def render[A](data: A)(using CT: ClassTag[A]): String = 
      getRendererFor[A].render(data)

    private def getRendererFor[A](using CT: ClassTag[A]): SingleTypeRenderer[A] =
      Option(registry.get(CT.runtimeClass)) match
        case Some(r) => r.asInstanceOf[SingleTypeRenderer[A]]
        case None => throw new IllegalArgumentException("No renderer in registry!")

  end RegistryRenderer

end DataRenderer
